@@ -1,3 +1,4 @@
+/// Tokens are the intermediate representation format in the markdown to html conversion
 #[derive(Debug, PartialEq)]
 pub enum Token {
     Plaintext(String),
@@ -8,7 +9,6 @@ pub enum Token {
     Bold(String),
     BoldItalic(String),
     LineBreak,
-    ParagraphBreak,
     Newline,
     HorizontalRule,
     Tab,
@@ -17,12 +17,36 @@ pub enum Token {
     Code(String),
     CodeBlock(String, String),
     BlockQuote(u8, String),
-    Image(String, String), // (Link, title)
+    Image(String, Option<String>), // (Link, title)
     Link(String, Option<String>, Option<String>), //(link, title, hover text)
     Detail(String, Vec<Token>),
-    Table(Vec<(Alignment, String)>, Vec<Vec<(Alignment, String)>>)
+    Table(Vec<(Alignment, String)>, Vec<Vec<(Alignment, Vec<Token>)>>),
+    TaskListItem(TaskBox, String),
+    Footnote(String, String),
 }
 
+/// Holds the possible states of a taskbox in a task list
+#[derive(Debug, PartialEq)]
+pub enum TaskBox {
+    Checked,
+    Unchecked,
+}
+
+impl Token {
+    fn is_usable_in_table(&self) -> bool {
+        match self {
+            Token::Code(_) => true,
+            Token::Link(_, _, _) => true,
+            Token::Bold(_) => true,
+            Token::Italic(_) => true,
+            Token::BoldItalic(_) => true,
+            Token::Plaintext(_) => true,
+            _ => false
+        }
+    }
+}
+
+/// Holds the alignment states for the table token
 #[derive(Debug, PartialEq, Clone)]
 pub enum Alignment {
     Left,
@@ -204,8 +228,11 @@ pub(crate) fn lex_backticks(char_iter: &mut std::iter::Peekable<std::str::Chars>
 }
 
 pub(crate) fn lex_newlines(char_iter: &mut std::iter::Peekable<std::str::Chars>) -> Result<Token, ParseError> {
-    consume_while_case_holds(char_iter, &|c| c == &'\n');
-    return Ok(Token::Newline);
+    let newlines = consume_while_case_holds(char_iter, &|c| c == &'\n');
+    match newlines.len() {
+        0..=1 => return Err(ParseError{content: newlines}),
+        _ => return Ok(Token::Newline)
+    }
 }
 
 pub(crate) fn lex_blockquotes(char_iter: &mut std::iter::Peekable<std::str::Chars>) -> Result<Token, ParseError> {
@@ -215,6 +242,7 @@ pub(crate) fn lex_blockquotes(char_iter: &mut std::iter::Peekable<std::str::Char
         _ => {return Err(ParseError{content: right_arrows})}
     }
     let s = consume_while_case_holds(char_iter, &|c| c != &'\n');
+    char_iter.next_if_eq(&'\n');
     Ok(Token::BlockQuote(right_arrows.len() as u8, s))
 }
 
@@ -226,7 +254,7 @@ pub(crate) fn lex_images(char_iter: &mut std::iter::Peekable<std::str::Chars>) -
     let link_result = lex_links(char_iter);
     match link_result {
         Err(e) => return Err(e),
-        Ok(Token::Link(link, title, _)) => return Ok(Token::Image(link, title.unwrap_or("".to_string()))),
+        Ok(Token::Link(link, title, _)) => return Ok(Token::Image(link, title)),
         _ => return Err(ParseError{content: "Non link token returned from lex_links".to_string()})
     }
 }
@@ -241,6 +269,37 @@ pub(crate) fn lex_links(char_iter: &mut std::iter::Peekable<std::str::Chars>) ->
         return Err(ParseError{content: "[".to_string()+&title})
     }
     char_iter.next();
+    // Parse footnotes big and small
+    if title.starts_with("^") && char_iter.peek() == Some(&':') {
+        char_iter.next();
+        let ref_id = title.strip_prefix("^").unwrap_or("");
+        let mut note_text = String::new();
+        loop {
+           note_text.push_str(&consume_while_case_holds(char_iter, &|c| c != &'\n'));
+           char_iter.next();
+           if char_iter.peek() != Some(&' ') && char_iter.peek() != Some(&'\t') {
+            break;
+           }
+           if char_iter.peek() == Some(&'\t') {
+            char_iter.next();
+            note_text.push('\n');
+            continue;
+           }
+           if char_iter.peek() == Some(&' ') {
+            let spaces = consume_while_case_holds(char_iter, &|c| c == &' ');
+            match spaces.len() {
+                2 | 4 => {note_text.push('\n');},
+                _ => {return Err(ParseError{content: "[^".to_string()+&ref_id.to_string()+&"]:".to_string()+&note_text+&spaces})},
+            }
+            continue
+           }
+           break;
+        }
+        if ref_id.contains(char::is_whitespace){
+            return Err(ParseError{content: "[^".to_string()+&ref_id.to_string()+&"]:".to_string()+&note_text})
+        }
+        return Ok(Token::Footnote(ref_id.to_string(), note_text.trim_start().to_string()));
+    }
     if char_iter.peek() != Some(&'(') {
         return Err(ParseError{content: "[".to_string()+&title+"]"})
     }
@@ -250,7 +309,7 @@ pub(crate) fn lex_links(char_iter: &mut std::iter::Peekable<std::str::Chars>) ->
         return Err(ParseError{content: "[".to_string()+&title+&"]".to_string()+&link})
     }
     if char_iter.peek() == Some(&')') {
-        char_iter.skip_while(|c| c != &'\n').next();
+        char_iter.next();
         return Ok(Token::Link(link, Some(title), None));
     }
     if char_iter.peek() == Some(&' ') {
@@ -264,13 +323,14 @@ pub(crate) fn lex_links(char_iter: &mut std::iter::Peekable<std::str::Chars>) ->
 pub(crate) fn lex_side_carrot(char_iter: &mut std::iter::Peekable<std::str::Chars>) -> Result<Token, ParseError> {
     match char_iter.peek() {
         Some(&'<') => {
+            char_iter.next();
             let s = consume_while_case_holds(char_iter, &|c| c != &'>');
             match char_iter.peek(){
-                Some(&'>') if s != "<details" => {
+                Some(&'>') if s != "details" => {
                     char_iter.next();
                     return Ok(Token::Link(s, None, None))
                 },
-                Some(&'>') if s == "<details" => {
+                Some(&'>') if s == "details" => {
                     char_iter.next();
                     if !char_iter.next_if_eq(&'\n').is_some(){
                         return Err(ParseError{content: s});
@@ -287,21 +347,22 @@ pub(crate) fn lex_side_carrot(char_iter: &mut std::iter::Peekable<std::str::Char
 }
 
 pub(crate) fn lex_plus_minus(char_iter: &mut std::iter::Peekable<std::str::Chars>) -> Result<Token, ParseError> {
-    let c = char_iter.next().unwrap();
-    if char_iter.next_if_eq(&' ').is_some(){
-        let s = consume_while_case_holds(char_iter, &|c| c != &'\n');
-        return Ok(Token::UnorderedListEntry(s))
+    let s = consume_while_case_holds(char_iter, &|c| c == &'-');
+    match s.len() {
+        3..=usize::MAX => { return Ok(Token::HorizontalRule)},
+        2 => {return Ok(Token::Plaintext(s))},
+        1 => {},
+        _ => {return Err(ParseError{content: "negative string length".to_string()})},
     }
-    match c {
-        '-' => {
-            let s = consume_while_case_holds(char_iter, &|c| c == &'-');
-            if s.chars().all(|x| x == '-') && char_iter.peek() == Some(&'\n'){
-                return Ok(Token::HorizontalRule)
-            } else {
-                return Ok(Token::Plaintext("-".to_string()+&s))
-            }
-        },
-        _ => return Ok(Token::Plaintext(c.to_string())),
+    let line = consume_while_case_holds(char_iter, &|c| c != &'\n');
+    if line.starts_with(" [ ] ") {
+        return Ok(Token::TaskListItem(TaskBox::Unchecked,line.strip_prefix(" [ ] ").unwrap_or("").to_string()))
+    } else if line.starts_with(" [x] ") {
+        return Ok(Token::TaskListItem(TaskBox::Checked,line.strip_prefix(" [x] ").unwrap_or("").to_string()))
+    } else if line.starts_with(" [X] ") {
+        return Ok(Token::TaskListItem(TaskBox::Checked,line.strip_prefix(" [X] ").unwrap_or("").to_string()))
+    } else {
+        return Ok(Token::Plaintext(s+&line))
     }
 }
 
@@ -410,7 +471,9 @@ pub(crate) fn lex_pipes(char_iter: &mut std::iter::Peekable<std::str::Chars>) ->
         .collect();
         let mut r = Vec::new();
         for e in elements.into_iter() {
-            r.push(e);
+            let mut inner_tokens = crate::lex(&e);
+            inner_tokens.retain(|token| token.is_usable_in_table());
+            r.push(inner_tokens);
         }
         rows.push(alignments.clone().into_iter().zip(r.into_iter()).collect());
     }
