@@ -1,5 +1,5 @@
 use mini_markdown::lex;
-use mini_markdown::lexer::Token;
+use mini_markdown::lexer::{Token, TaskBox};
 
 #[test]
 fn test_lex() {
@@ -31,7 +31,7 @@ fn test_lex() {
         ("I just love *italic text*.", vec![Token::Plaintext("I just love ".to_string()), Token::Italic("italic text".to_string()), Token::Plaintext(".".to_string())]),
         ("I just love _italic text_.", vec![Token::Plaintext("I just love ".to_string()), Token::Italic("italic text".to_string()), Token::Plaintext(".".to_string())]),
         ("I just love *italic text_.", vec![Token::Plaintext("I just love ".to_string()), Token::Italic("italic text".to_string()), Token::Plaintext(".".to_string())]),
-        ("I just\n love *italic\n text_.", vec![Token::Plaintext("I just".to_string()), Token::Newline, Token::Plaintext(" love ".to_string()), Token::Italic("italic\n text".to_string()), Token::Plaintext(".".to_string())]),
+        ("I just\n love *italic\n text_.", vec![Token::Plaintext("I just\n love ".to_string()), Token::Italic("italic\n text".to_string()), Token::Plaintext(".".to_string())]),
     ]);
     tests.extend(vec![
         ("I just love ***bold italic text***.", vec![Token::Plaintext("I just love ".to_string()), Token::BoldItalic("bold italic text".to_string()), Token::Plaintext(".".to_string())]),
@@ -49,8 +49,7 @@ fn test_lex() {
         vec![
             Token::Plaintext("Text attributes ".to_string()), 
             Token::Italic("italic".to_string()), 
-            Token::Plaintext(", ".to_string()),
-            Token::Newline, 
+            Token::Plaintext(", \n".to_string()),
             Token::Bold("bold".to_string()), 
             Token::Plaintext(", ".to_string()), 
             Token::Code("monospace".to_string()),
@@ -59,12 +58,80 @@ fn test_lex() {
             Token::Plaintext(" for italic text.".to_string()),
         ])
     ]);
+    tests.extend(vec![
+        ("![alt](https://example.com/foo.jpeg)", vec![Token::Image("https://example.com/foo.jpeg".to_string(), Some("alt".to_string()))]),
+        ("![alt]()", vec![Token::Image("".to_string(), Some("alt".to_string()))]),
+        ("Some test text [^1]", vec![Token::Plaintext("Some test text [^1]".to_string())]),
+        ("[^1]: First footnote", vec![Token::Footnote("1".to_string(), "First footnote".to_string())]),
+        ("[^HUGE]: Big footnote", vec![Token::Footnote("HUGE".to_string(), "Big footnote".to_string())]),
+        ("[^BORK ED]: Big footnote", vec![Token::Plaintext("[^BORK ED]: Big footnote".to_string())]),
+
+    ]);
     tests.extend(vec![
         ("¯\\\\\\_(ツ)\\_/¯", vec![Token::Plaintext("¯\\_(ツ)_/¯".to_string())]),
         ("\\_test\\_", vec![Token::Plaintext("_test_".to_string())]),
         ("\\*escaping\\_", vec![Token::Plaintext("*escaping_".to_string())]),
         ("\\>controls\\<", vec![Token::Plaintext(">controls<".to_string())])
     ]);
+    tests.extend(vec![
+        ("---", vec![Token::HorizontalRule]),
+        ("-----", vec![Token::HorizontalRule]),
+        ("--", vec![Token::Plaintext("--".to_string())]),
+        ("- [ ] Unchecked box", vec![Token::TaskListItem(TaskBox::Unchecked, "Unchecked box".to_string())]),
+        ("- [x] Checked box", vec![Token::TaskListItem(TaskBox::Checked, "Checked box".to_string())]),
+        ("- [X] Also a checked box", vec![Token::TaskListItem(TaskBox::Checked, "Also a checked box".to_string())]),
+        ("- [X]Not a checked box", vec![Token::Plaintext("- [X]Not a checked box".to_string())]),
+        ("- [X] A checked box\n- [X] Also a checked box", vec![Token::TaskListItem(TaskBox::Checked, "A checked box".to_string()), Token::Plaintext("\n".to_string()), Token::TaskListItem(TaskBox::Checked, "Also a checked box".to_string())]),
+    ]);
+    for test in tests.iter(){
+        let tokens = lex(test.0);
+        assert_eq!(&tokens[..], &test.1[..]);
+    }
+}
+
+#[test]
+fn test_blockquote_lex() {
+    let mut tests = Vec::new();
+    tests.extend(vec![
+        ("> ", vec![Token::BlockQuote(1, "".to_string())]),
+        ("> \n>> text", vec![Token::BlockQuote(1, "".to_string()), Token::BlockQuote(2, "text".to_string())]),
+        ("> text\n> \n>> more text", vec![Token::BlockQuote(1, "text".to_string()) ,Token::BlockQuote(1, "".to_string()), Token::BlockQuote(2, "more text".to_string())]),
+
+    ]);
+
+    for test in tests.iter(){
+        let tokens = lex(test.0);
+        assert_eq!(&tokens[..], &test.1[..]);
+    }
+}
+
+#[test]
+fn test_footnote_lex() {
+    let mut tests = Vec::new();
+    tests.extend(vec![
+        ("[^1]: Footnote #1", vec![Token::Footnote("1".to_string(), "Footnote #1".to_string())]),
+        ("[^1]: Footnote #1\n  with a second line", vec![Token::Footnote("1".to_string(), "Footnote #1\nwith a second line".to_string())]),
+        ("[^1]: Footnote #1\n\twith a second line", vec![Token::Footnote("1".to_string(), "Footnote #1\nwith a second line".to_string())]),
+        ("[^1]: Footnote #1\n    with a second line", vec![Token::Footnote("1".to_string(), "Footnote #1\nwith a second line".to_string())]),
+        ("[^1]: Footnote #1\n    with a second line\n\tand a third line", vec![Token::Footnote("1".to_string(), "Footnote #1\nwith a second line\nand a third line".to_string())]),
+    ]);
+
+    for test in tests.iter(){
+        let tokens = lex(test.0);
+        assert_eq!(&tokens[..], &test.1[..]);
+    }
+}
+
+#[test]
+fn test_link_lex(){
+    let mut tests = Vec::new();
+    tests.extend(vec![
+        ("another (See [Sewer Shark](https://en.wikipedia.org/wiki/Sewer_Shark)). Video", 
+        vec![Token::Plaintext("another (See ".to_string()), Token::Link("https://en.wikipedia.org/wiki/Sewer_Shark".to_string(), Some("Sewer Shark".to_string()), None), Token::Plaintext("). Video".to_string())]),
+        ("r [Distant Worlds](https://www.youtube.com/watch?v=yd3KYOei8o4) a",
+        vec![Token::Plaintext("r ".to_string()), Token::Link("https://www.youtube.com/watch?v=yd3KYOei8o4".to_string(), Some("Distant Worlds".to_string()), None), Token::Plaintext(" a".to_string())])
+    ]);
+
     for test in tests.iter(){
         let tokens = lex(test.0);
         assert_eq!(&tokens[..], &test.1[..]);
